@@ -1,3 +1,4 @@
+from contextlib import contextmanager
 from functools import partial
 import json
 import re
@@ -16,7 +17,9 @@ except ImportError:
 from ._binary import KTBinaryProtocol
 from ._binary import TTBinaryProtocol
 from ._binary import decode
+from ._binary import dict_to_table
 from ._binary import encode
+from ._binary import table_to_dict
 from .exceptions import ImproperlyConfigured
 from .exceptions import KyotoTycoonError
 from .exceptions import ProtocolError
@@ -30,7 +33,9 @@ KT_JSON = 'json'
 KT_MSGPACK = 'msgpack'
 KT_NONE = 'none'
 KT_PICKLE = 'pickle'
-KT_SERIALIZERS = set((KT_BINARY, KT_JSON, KT_MSGPACK, KT_NONE, KT_PICKLE))
+TT_TABLE = 'table'
+KT_SERIALIZERS = set((KT_BINARY, KT_JSON, KT_MSGPACK, KT_NONE, KT_PICKLE,
+                      TT_TABLE))
 
 
 class BaseClient(object):
@@ -61,6 +66,9 @@ class BaseClient(object):
             self._encode_value = partial(pickle.dumps,
                                          protocol=pickle.HIGHEST_PROTOCOL)
             self._decode_value = pickle.loads
+        elif self._serializer == TT_TABLE:
+            self._encode_value = dict_to_table
+            self._decode_value = table_to_dict
         else:
             raise ImproperlyConfigured('unrecognized serializer "%s" - use one'
                                        ' of: %s' % (self._serializer,
@@ -75,8 +83,20 @@ class BaseClient(object):
             self._script_runner = ScriptRunner(self)
         return self._script_runner
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._protocol.checkin()
+
+    def checkin(self):
+        self._protocol.checkin()
+
     def close(self):
-        return self._protocol.close()
+        self._protocol.close()
+
+    def close_idle(self, n=60):
+        self._protocol.close_idle(n)
 
 
 class ScriptRunner(object):
@@ -109,6 +129,10 @@ class KyotoTycoon(BaseClient):
             encode_value=self._encode_value,
             decode_value=self._decode_value)
 
+    def close(self):
+        self._protocol.close()
+        self._protocol_http.close()
+
     def get(self, key, db=None):
         db = self._default_db if db is None else db
         return self._protocol.get(key, db)
@@ -153,6 +177,20 @@ class KyotoTycoon(BaseClient):
     def report(self):
         return self._protocol_http.report()
 
+    def ulog_list(self):
+        return self._protocol_http.ulog_list()
+
+    def ulog_remove(self, max_dt):
+        return self._protocol_http.ulog_remove(max_dt)
+
+    def synchronize(self, hard=False, db=None):
+        db = self._default_db if db is None else db
+        return self._protocol_http.synchronize(hard, db)
+
+    def vacuum(self, step=0, db=None):
+        db = self._default_db if db is None else db
+        return self._protocol_http.vacuum(step, db)
+
     def add(self, key, value, db=None, expire_time=None):
         db = self._default_db if db is None else db
         return self._protocol_http.add(key, value, db, expire_time)
@@ -218,19 +256,30 @@ class KyotoTycoon(BaseClient):
     def __len__(self):
         return int(self.status(self._default_db)['count'])
 
+    def count(self, db=None):
+        db = self._default_db if db is None else db
+        return int(self.status(db)['count'])
+
     def match_prefix(self, prefix, max_keys=None, db=None):
+        db = self._default_db if db is None else db
         return self._protocol_http.match_prefix(prefix, max_keys, db)
 
     def match_regex(self, regex, max_keys=None, db=None):
+        db = self._default_db if db is None else db
         return self._protocol_http.match_regex(regex, max_keys, db)
 
     def match_similar(self, origin, distance=None, max_keys=None, db=None):
+        db = self._default_db if db is None else db
         return self._protocol_http.match_similar(origin, distance, max_keys,
                                                  db)
 
     def keys(self, db=None):
+        db = self._default_db if db is None else db
         return self.match_prefix('', db=db)
 
+    def __iter__(self):
+        return iter(self.keys())
+
     @property
     def size(self):
         return int(self.status(self._default_db)['size'])
@@ -308,6 +357,9 @@ class TokyoTyrant(BaseClient):
     def exists(self, key):
         return self._protocol.vsiz(key)
 
+    def length(self, key):
+        return self._protocol.vsiz(key)
+
     def incr(self, key, n=1):
         return self._protocol.addint(key, n)
 
@@ -344,6 +396,9 @@ class TokyoTyrant(BaseClient):
     def optimize(self):
         return self._protocol.misc('optimize', [])
 
+    def synchronize(self):
+        return self._protocol.sync()
+
     def clear_cache(self):
         return self._protocol.misc('cacheclear', [])
 
@@ -362,7 +417,7 @@ class TokyoTyrant(BaseClient):
         return {} if rv is True else rv
 
     def iter_from(self, start_key):
-        self._protocol.misc('iterinit', [item.start])
+        self._protocol.misc('iterinit', [start_key])
         accum = {}
         while True:
             kv = self._protocol.misc('iternext', [])
